/*
 * Lancaster University
 * Computing Department
 * 
 * Created by Eduardo Figueiredo
 * Date: 11 Aug 2007
 * 
 */
package ubc.midp.mobilephoto.core.ui.controller;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.List;

import ubc.midp.mobilephoto.core.ui.MainUIMidlet;
import ubc.midp.mobilephoto.core.ui.datamodel.AlbumData;
import ubc.midp.mobilephoto.core.ui.datamodel.MediaData;
import ubc.midp.mobilephoto.core.ui.screens.AlbumListScreen;
import ubc.midp.mobilephoto.core.ui.screens.MediaListScreen;
import ubc.midp.mobilephoto.core.util.Constants;
import lancs.midp.ptolemy.exceptionblocks.controllerAspectEH.events.*;
import lancs.midp.mobilephoto.optional.sorting.CountViewAspect.events.*;
import lancs.midp.mobilephoto.alternative.music.events.*;

/**
 * @author Eduardo Figueiredo
 *
 */
public class MediaListController extends AbstractController {

	/**
	 * @param midlet
	 * @param nextController
	 * @param albumData
	 * @param albumListScreen
	 */
	public MediaListController(MainUIMidlet midlet, AlbumData albumData, AlbumListScreen albumListScreen) {
		super(midlet, albumData, albumListScreen);
	}

	/* (non-Javadoc)
	 * @see ubc.midp.mobilephoto.core.ui.controller.ControllerInterface#handleCommand(java.lang.String)
	 */
	public boolean handleCommand(Command command) {
		String label = command.getLabel();
		/** Case: Select PhotoAlbum to view **/
		if (label.equals("Select")) {
			// Get the name of the PhotoAlbum selected, and load image list from
			// RecordStore
			List down = (List) Display.getDisplay(midlet).getCurrent();
			ScreenSingleton.getInstance().setCurrentStoreName(down.getString(down.getSelectedIndex()));
			showMediaList(getCurrentStoreName());
			ScreenSingleton.getInstance().setCurrentScreenName( Constants.IMAGELIST_SCREEN);
			return true;
		}
		
		return false;
	}

    /**
     * Show the list of images in the record store
	 * TODO: Refactor - Move this to ImageList class
	 */
	public void showMediaList(String recordName) {
		announce ShowImageListEvent(midlet)
		{
			if (recordName == null)
				recordName = getCurrentStoreName();
			
			MediaController mediaController = new MediaController(midlet, getAlbumData(), (AlbumListScreen)getAlbumListScreen());
			mediaController.setNextController(this);
			
			MediaListScreen imageList = announce MediaListScreenCreatedEvent (this) {
				return new MediaListScreen();
			};
			imageList.setCommandListener(mediaController);
			
			//Command selectCommand = new Command("Open", Command.ITEM, 1);
			imageList.initMenu();
			
			MediaData[] images = null;
			
			images = getAlbumData().getMedias(recordName);
			
			if (images==null) return;
			
			announce AppendImagesEvent(images)
			{
				//loop through array and add labels to list
				for (int i = 0; i < images.length; i++) {
					if (images[i] != null) {
						//Add album name to menu list
						
						imageList.append(images[i].getMediaLabel(), null);						
					}
				}
			}

			setCurrentScreen(imageList);
			//currentMenu = "list";
		}
	}
	
	public void postCommand(Command command) {
        System.out.println("BaseController::postCommand - Current controller is: " + this.getClass().getName());
        //If the current controller cannot handle the command, pass it to the next
        //controller in the chain.
        if (handleCommand(command) == false) {
        	ControllerInterface next = getNextController();
            if (next != null) {
                System.out.println("Passing to next controller in chain: " + next.getClass().getName());
                next.postCommand(command);
            } else {
                System.out.println("BaseController::postCommand - Reached top of chain. No more handlers for command: " + command);
            }
        }
	}
}
